from django.contrib.admin.filters import (AllValuesFieldListFilter,
                                          RelatedFieldListFilter,
                                          SimpleListFilter,)
from django.db.models.query_utils import Q
from django.utils.encoding import smart_text
from django.utils.translation import ugettext as _

try:
    from django.db.models.fields.related import ForeignObjectRel
except ImportError:
    from django.db.models.related import RelatedObject as ForeignObjectRel


class AllValuesComboFilter(AllValuesFieldListFilter):
    template = 'adminfilters/combobox.html'


class AllValuesRadioFilter(AllValuesFieldListFilter):
    template = 'adminfilters/fieldradio.html'


class RelatedFieldComboFilter(RelatedFieldListFilter):
    template = 'adminfilters/fieldcombobox.html'


class RelatedFieldRadioFilter(RelatedFieldListFilter):
    template = 'adminfilters/fieldradio.html'


class RelatedFieldCheckBoxFilter(RelatedFieldListFilter):
    template = 'adminfilters/fieldcheckbox.html'

    def __init__(self, field, request, params, model, model_admin, field_path):
        self.model_admin = model_admin
        super(RelatedFieldCheckBoxFilter, self).__init__(field, request, params, model, model_admin, field_path)
        self.lookup_val = request.GET.getlist(self.lookup_kwarg, [])

    def queryset(self, request, queryset):
        if not len(self.lookup_val):
            return queryset

        filters = []
        for val in self.lookup_val:
            filters.append(Q(**{self.lookup_kwarg: val}))

        query = filters.pop()
        for item in filters:
            query |= item

        return queryset.filter(query)

    def choices(self, cl):
        try:
            from django.contrib.admin.views.main import EMPTY_CHANGELIST_VALUE
        except ImportError:
            EMPTY_CHANGELIST_VALUE = self.model_admin.get_empty_value_display()

        yield {
            'selected': not len(self.lookup_val) and not self.lookup_val_isnull,
            'query_string': cl.get_query_string(
                {},
                [self.lookup_kwarg, self.lookup_kwarg_isnull]),
            'display': _('All'),
        }
        yield {
            'selected': self.lookup_val_isnull,
            'query_string': cl.get_query_string({self.lookup_kwarg_isnull: 1},
                                                [self.lookup_kwarg, self.lookup_kwarg_isnull]),
            'display': _('None'),
        }
        for pk_val, val in self.lookup_choices:
            yield {
                'selected': smart_text(pk_val) in self.lookup_val,
                'query_string': cl.get_query_string(
                    {
                        self.lookup_kwarg: pk_val,
                    },
                    [self.lookup_kwarg_isnull]),
                'display': val,
            }
        if (isinstance(self.field, ForeignObjectRel) and
                self.field.field.null or
                hasattr(self.field, 'rel') and
                self.field.null):
            yield {
                'selected': bool(self.lookup_val_isnull),
                'query_string': cl.get_query_string(
                    {
                        self.lookup_kwarg_isnull: 'True',
                    },
                    [self.lookup_kwarg]),
                'display': EMPTY_CHANGELIST_VALUE,
            }


class StartWithFilter(SimpleListFilter):
    prefixes = None
    lookup_val = None

    def lookups(self, request, model_admin):
        return self.prefixes

    def queryset(self, request, queryset):
        if not self.value():
            return queryset
        if self.value() == '--':
            k = [prefix for prefix, label in self.prefixes]
            query = Q()
            for el in k:
                query |= Q(codename__startswith=el)
            return queryset.exclude(query)
        else:
            return queryset.filter(codename__startswith=self.value())


class PermissionPrefixFilter(StartWithFilter):
    title = 'Permission'
    parameter_name = 'perm'
    prefixes = (('add', 'Add'), ('change', 'Change'), ('delete', 'Delete'), ('--', 'Others'))



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2015-12-17 18:36
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('demoapp', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='DemoModelField',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('char', models.CharField(max_length=255)),
                ('integer', models.IntegerField()),
                ('logic', models.BooleanField()),
                ('null_logic', models.NullBooleanField()),
                ('date', models.DateField()),
                ('datetime', models.DateTimeField()),
                ('time', models.TimeField()),
                ('decimal', models.DecimalField(decimal_places=3, max_digits=10)),
                ('email', models.EmailField(max_length=254)),
                ('float', models.FloatField()),
                ('bigint', models.BigIntegerField()),
                ('generic_ip', models.GenericIPAddressField()),
                ('url', models.URLField()),
                ('text', models.TextField()),
                ('unique', models.CharField(max_length=255, unique=True)),
                ('nullable', models.CharField(max_length=255, null=True)),
                ('blank', models.CharField(blank=True, max_length=255, null=True)),
                ('not_editable', models.CharField(blank=True, editable=False, max_length=255, null=True)),
                ('choices', models.IntegerField(choices=[(1, b'Choice 1'), (2, b'Choice 2'), (3, b'Choice 3')])),
            ],
        ),
        migrations.CreateModel(
            name='DemoModel_IntersectionFieldListFilter',
            fields=[
            ],
            options={
                'proxy': True,
                'verbose_name': 'IntersectionFieldListFilter',
            },
            bases=('demoapp.demomodel',),
        ),
        migrations.CreateModel(
            name='DemoModel_RelatedFieldCheckBoxFilter',
            fields=[
            ],
            options={
                'proxy': True,
                'verbose_name': 'RelatedFieldCheckBoxFilter',
            },
            bases=('demoapp.demomodel',),
        ),
        migrations.CreateModel(
            name='DemoModel_RelatedFieldRadioFilter',
            fields=[
            ],
            options={
                'proxy': True,
                'verbose_name': 'RelatedFieldRadioFilter',
            },
            bases=('demoapp.demomodel',),
        ),
        migrations.CreateModel(
            name='DemoModel_UnionFieldListFilter',
            fields=[
            ],
            options={
                'proxy': True,
                'verbose_name': 'UnionFieldListFilter',
            },
            bases=('demoapp.demomodel',),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='DemoModel',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='DemoRelated',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=255)),
            ],
        ),
        migrations.AddField(
            model_name='demomodel',
            name='demo_related',
            field=models.ForeignKey(to='demoapp.DemoRelated'),
        ),
    ]



#!/usr/bin/env python
import codecs
import imp
import os
import sys

from setuptools import find_packages, setup

ROOT = os.path.realpath(os.path.join(os.path.dirname(__file__)))
init = os.path.join(ROOT, 'src', 'adminfilters', '__init__.py')

app = imp.load_source('adminfilters', init)
reqs = 'install.py%d.pip' % sys.version_info[0]


def read(*files):
    content = ''
    for f in files:
        content += codecs.open(os.path.join(ROOT, 'src',
                                            'requirements', f), 'r').read()
    return content

install_requires = read('install.any.pip', reqs)
tests_requires = read('testing.pip')
dev_requires = tests_requires + read('develop.pip')


setup(name=app.NAME,
      version=app.get_version(),
      url='https://github.com/saxix/django-adminfilters',
      download_url='https://github.com/saxix/django-adminfilters',
      author='sax',
      author_email='sax@os4d.org',
      description="Extra filters for django admin site",
      license='MIT',
      package_dir={'': 'src'},
      packages=find_packages('src'),
      include_package_data=True,
      # install_requires=fread(reqs),
      # tests_require=tests_require,
      extras_require={
          'test': tests_requires,
          'dev': dev_requires
      },
      platforms=['any'],
      command_options={
          'build_sphinx': {
              'version': ('setup.py', app.VERSION),
              'release': ('setup.py', app.VERSION)}
      },
      classifiers=[
          'Environment :: Web Environment',
          'Framework :: Django',
          'Operating System :: OS Independent',
          'Programming Language :: Python',
          'Intended Audience :: Developers'],
      long_description=codecs.open('README.rst').read()
      )
